--[[ *** Create Ace2 AddOn *** ]]
RaidSpy = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDebug-2.0", "AceConsole-2.0", "AceDB-2.0")
local L = AceLibrary("AceLocale-2.2"):new("RaidSpy")

--[[ *** Initialize the AddOn *** ]]
function RaidSpy:OnInitialize()
	--[[ Register a DB for saving ]]
	self:RegisterDB("RaidSpyDB")
	self:RegisterDefaults('profile', {
		item = true,
		dura = true,
		reag = true,
		resi = true,
		chatR = true,
		erroR = true,
	})
  
  self:BuildOpts()
  
  --[[ Register chat commands ]]    
  self:RegisterChatCommand(L["consolecommands"], self.opts)
  
  --[[ Tell user we loaded ]]
	self:Print(L["translator"])
end

--[[ *** Do stuff when enabled? *** ]]
function RaidSpy:OnEnable()
	--[[ Register Events for example ]]
  self:RegisterEvent("CHAT_MSG_ADDON")
end

--[[ *** Do stuff when disabled? *** ]]
function RaidSpy:OnDisable()
  --[[ Unregister Events for example ]]
  self:UnregisterAllEvents()
end

--[[ *** EVENTS *** ]]
function RaidSpy:CHAT_MSG_ADDON()
	if ( arg1 == "CTRA" and arg3 == "RAID" ) then
		local msg = string.gsub(arg2, "%$", "s");
		msg = string.gsub(msg, "", "S");
		if (strsub(msg, strlen(msg)-7) == " ...hic!") then
			msg = strsub(msg, 1, strlen(msg)-8);
		end
		
		if (string.find(msg, "#")) then
			for k, v in pairs(self:Split(msg, "#")) do
				self:CheckEvent(arg4, v)
			end
		else
			self:CheckEvent(arg4, msg)
		end
	end
end

--[[ *** Functions *** ]]
function RaidSpy:CheckEvent(nick, msg)		
	if (msg == "RSTC" and self.db.profile.resi) then -- Resists
		self:Report(nick, L["ResistanceText"])
	elseif (msg == "REAC" and self.db.profile.reag) then -- Reagents
		self:Report(nick, L["ReagentsText"])
	elseif (msg == "DURC" and self.db.profile.dura) then --Durabillity
		self:Report(nick, L["DurabilityText"])
	elseif (string.find(msg, "^ITMC ") and self.db.profile.item) then -- Items
		local _, _, itemName = string.find(msg, "^ITMC (.+)$")
		if (itemName) then
			self:Report(nick, (string.gsub(L["ItemText"], "{i}", itemName)))
		end
	end
end

function RaidSpy:Report(nick, msg)
	if ((nick == UnitName("player")) and (not self.debugging)) then return end
	if (self.db.profile.chatR) then
		self:Print(nick.." "..msg)
	end
	if (self.db.profile.erroR) then
		UIErrorsFrame:AddMessage(L["AddOnErrorMessagePrefix"]..nick.." "..msg)
	end
end

function RaidSpy:Split(text, delimiter)
	local list = {}
  local pos = 1
  if strfind("", delimiter, 1) then -- this would result in endless loops
    error("delimiter matches empty string!")
  end
  while 1 do
    local first, last = strfind(text, delimiter, pos)
    if first then -- found?
      tinsert(list, strsub(text, pos, first-1))
      pos = last+1
    else
      tinsert(list, strsub(text, pos))
      break
    end
  end
  return list
end

function RaidSpy:PerformItemCheck(item)
	if (not UnitInRaid("player")) then
		self:Print(L["UnitNotInRaid"])
		return
	end
	if (IsAddOnLoaded("oRA2")) then
		-- IsRaidOfficer() -- IsRaidLeader()
		oRALItem:PerformItemCheck(item)
	elseif (IsAddOnLoaded("CT_RaidAssist")) then
		SlashCmdList["RAITEM"](item)
	else
		self:Print(L["cantdoitemcheck"])
	end
end

function RaidSpy:BuildOpts()
	self.opts = {
	  type = "group",
	  args = {
	  	header = {
				type = "header",
				name = L["addonname"],
				icon = "Interface\\Icons\\INV_Misc_Spyglass_03",
				iconHeight = 16,
				iconWidth = 16,
				order = 1,
			},
			headerspacer = {
				type = "header",
				order = 2,
			},
			typespacer = {
				type = "header",
				order = 6,
			},
			checksspacer = {
				type = "header",
				order = 8,
			},
			checks = {
				type = "group",
				name = L["predefinedChecks_name"],
				desc = L["predefinedChecks_desc"],
				order = 9,
				args = {
					std = {
						type = "group",
						name = L["predefinedChecks_std_name"],
						desc = L["predefinedChecks_std_desc"],
						order = 1,
						args = { },
					},
					zg = {
						type = "group",
						name = L["predefinedChecks_zg_name"],
						desc = L["predefinedChecks_zg_desc"],
						order = 2,
						args = { },
					},
					mc = {
						type = "group",
						name = L["predefinedChecks_mc_name"],
						desc = L["predefinedChecks_mc_desc"],
						order = 2,
						args = { },
					},
					bwl = {
						type = "group",
						name = L["predefinedChecks_bwl_name"],
						desc = L["predefinedChecks_bwl_desc"],
						order = 2,
						args = { },
					},
					aq = {
						type = "group",
						name = L["predefinedChecks_aq_name"],
						desc = L["predefinedChecks_aq_desc"],
						order = 2,
						args = { },
					},
					naxx = {
						type = "group",
						name = L["predefinedChecks_naxx_name"],
						desc = L["predefinedChecks_naxx_desc"],
						order = 2,
						args = { },
					},
				},
			},
	  },
	}
	
	for k,v in pairs(self.db.profile) do
		self.opts.args[L[k.."_cmd"]] = {
			type = "toggle",
      name = L[k.."_name"],
      desc = L[k.."_desc"],
      get = function()
        return self.db.profile[k]
      end,
      set = function()
        self.db.profile[k] = not self.db.profile[k]
      end,
      order = tonumber(L[k.."_order"]),
		}
	end
	
	self.opts.args.checks.args[L["Hearthstone_name"]] = {
		type = "execute",
		name = "["..L["Hearthstone_name"].."]",
		desc = L["Hearthstone_desc"],
	  func = function() RaidSpy:PerformItemCheck(L["Hearthstone_name"]) end,
	}
	
	for k,_ in pairs(L["predefinedCheckItems"]) do
		for _,v in pairs(L["predefinedCheckItems"][k]) do
			local orderNum = 1
			local itemName = v
			if (string.find(itemName, "|")) then
				orderNum = tonumber(self:Split(itemName, "|")[1])
				itemName = self:Split(itemName, "|")[2]
			end
			self.opts.args.checks.args[k].args[string.gsub(itemName, "[^A-Za-z1-9_]", "")] = {
				type = "execute",
				name = "["..itemName.."]",
				desc = L["Run a check for: "]..itemName,
			  func = function() RaidSpy:PerformItemCheck(itemName) end,
			  order = orderNum
			}
		end
	end
end